from typing import Generator
from json import loads

from openai import OpenAI
from dotenv import load_dotenv, find_dotenv
from openai.types.chat import ChatCompletion
from openai._streaming import Stream
from zkappumstad.runners import RunnerMessage, ToolMessage, StateChange, StreamMessage

from zkappumstad.tools import (
    Tool,
    doc_tool,
    code_tool,
    project_tool,
    issue_tool,
    state_change_tool,
)
from zkappumstad.prompt import SYSTEM_PROMPT

load_dotenv(find_dotenv(".env.local"), override=True)

client = OpenAI()
tools: dict[str, Tool] = {
    tool.name: tool
    for tool in [doc_tool, code_tool, project_tool, issue_tool, state_change_tool]
}


def create_completion(history, message) -> Generator[RunnerMessage, None, None]:
    while True:
        try:
            history.append({"role": "user", "content": message})
            chat_completion: Stream[ChatCompletion] = client.chat.completions.create(
                messages=[
                    {"role": "system", "content": SYSTEM_PROMPT},
                    *history,
                ],
                model="gpt-4-1106-preview",
                temperature=0.2,
                stream=True,
                functions=[tool.description for tool in tools.values()],
                function_call="auto",
            )

            if not isinstance(chat_completion, Stream):
                continue

            part = next(chat_completion)

            if part.choices[0].delta.function_call:
                function_name = part.choices[0].delta.function_call.name
                tool = tools[function_name]

                yield ToolMessage(tool.message, "TOOL_MESSAGE")
                args = "".join(
                    list(
                        part.choices[0].delta.function_call.arguments
                        for part in chat_completion
                        if part.choices[0].delta.function_call
                    )
                )
                history.append(
                    {
                        "role": "assistant",
                        "content": None,
                        "function_call": {"name": function_name, "arguments": args},
                    }
                )

                args = loads(args)
                result = tool.function(**args)
                if isinstance(result, StateChange):
                    yield result
                    break
                history.append(
                    {"role": "function", "name": function_name, "content": result}
                )

                continue

            yield StreamMessage(part.choices[0].delta.content or "", "STREAM_MESSAGE")
            for part in chat_completion:
                yield StreamMessage(
                    part.choices[0].delta.content or "", "STREAM_MESSAGE"
                )
            break

        except Exception as e:
            print(e)
            print(e.with_traceback())
            return None


if __name__ == "__main__":
    create_completion([], "What's the weather in San Francisco?")



import subprocess
from .tool import Tool

function_description = {
    "name": "run_command",
    "description": "The function can be used to run terminal commands in the initial_project folder.",
    "parameters": {
        "type": "object",
        "properties": {
            "command_type": {
                "type": "string",
                "enum": ["TEST", "HELP", "BUILD"],
                "description": "The type of command.",
            },
        },
        "required": ["command_type"],
    },
}

function_messages = "Running command..."

basedir = "initial_project"
basedir_contracts = "initial_project/contracts/"
basedir_src = "initial_project/contracts/src"


def run_tool(command_type=""):
    try:
        if command_type == "TEST":
            result = subprocess.run(
                ["npm", "run", "test"],
                check=True,
                capture_output=True,
                text=True,
                cwd=basedir_contracts,
            )
            output = result.stdout + result.stderr
            print(output)
            return output
        elif command_type == "HELP":
            result = subprocess.run(
                ["zk", "--help"],
                check=True,
                text=True,
                capture_output=True,
                cwd=basedir_contracts,
            )
            output = result.stdout + result.stderr
            print(output)
            return output
        elif command_type == "BUILD":
            result = subprocess.run(
                ["npm", "run", "build"],
                check=False,
                text=True,
                capture_output=True,
                cwd=basedir_contracts,
            )
            print("____STDOUT____")
            print(result.stdout)
            print("____STDERR____")
            print(result.stderr)
            return result.stdout, result.stderr

    except subprocess.CalledProcessError as e:
        print(f"Command failed with error: {e}")
        print(e.stderr)
        output = e.stdout + e.stderr
        return output
    except Exception as e:
        print(f"An error occurred: {e}")


command_tool = Tool(
    name="run_command",
    description=function_description,
    message=function_messages,
    function=run_tool,
)



import os
from .tool import Tool
from zkappumstad.utils import fade_in_text

function_description = {
    "name": "write_code",
    "description": "Always run this function after using other functions. This tool provides the generation of files for zkApp projects and their tests. Use this function when AI wants to generate README files, code files for existing zkApps project such as smart contracts and their tests or user wants to write by you. The generated code aligns with the user's specifications for smart contract functionalities within the zkApps framework. This tool is ideal for developers seeking to streamline their zkApp development process, ensuring rapid prototyping and consistency in code structure.",
    "parameters": {
        "type": "object",
        "properties": {
            "contract_name": {
                "type": "string",
                "description": "The name of the contract related the project",
            },
            "code": {
                "type": "string",
                "description": "The typescript code for zkApps smart contracts and their tests to write. Or markdown code for readme file. Write the code completely, no placeholder.",
            },
            "file_type": {
                "type": "string",
                "enum": ["CONTRACT", "TEST", "README"],
                "description": "The type of the file to write.",
            },
        },
        "required": ["code", "file_type"],
    },
}

function_messages = "Writing code to file..."

basedir = "initial_project"
basedir_src = "initial_project/contracts/src"


def run_tool(contract_name="", code="", file_type=""):
    try:
        if file_type == "TEST":
            filename = f"{contract_name}.test.ts"
        elif file_type == "CONTRACT":
            filename = f"{contract_name}.ts"
        if file_type == "README":
            filename = "README.md"
            file_path = os.path.join(basedir, filename)
        else:
            file_path = os.path.join(basedir_src, filename)

        os.makedirs(os.path.dirname(file_path), exist_ok=True)

        with open(file_path, "w") as file:
            file.write(code)
            fade_in_text(f"Code written successfully to {file_path}!", "bold green")

        return code

    except Exception as e:
        print(f"An error occurred: {e}")


writer_tool = Tool(
    name="write_code",
    description=function_description,
    message=function_messages,
    function=run_tool,
)



import requests

from .tool import Tool
from .api_query import query_index

function_description = {
    "name": "search_for_code_context",
    "description": "Always firstly use this tool for any code related requests. Understand and benchmark the smart contract structure. Search for context from code examples in the documentations, use this tool to retrieve context about code, when you need a reference code, Second most used tool.",
    "parameters": {
        "type": "object",
        "properties": {
            "query": {
                "type": "string",
                "description": "The query to search for. 1-3 sentences are enough. English only.",
            },
        },
        "required": ["query"],
    },
}

function_messages = "Fetching context about code snippets..."


def run_tool(query=""):
    """Run the query tool with the given parameters."""
    matches = query_index(tool="search_for_code_context", query=query)
    if not matches:
        return "No matches found in the database for the query: " + query

    return matches


code_tool = Tool(
    name="search_for_code_context",
    description=function_description,
    message=function_messages,
    function=run_tool,
)



import requests

from .tool import Tool
from .api_query import query_project


function_description = {
    "name": "search_for_project_context",
    "description": "Search for context for example zkApp projects or developer tools on MINA Protocol, use this tool to retrieve project code examples about zkApps, o(1)js, MINA smart contracts. You will need this tool when user asks smart contracts or zkApp projects.",
    "parameters": {
        "type": "object",
        "properties": {
            "query": {
                "type": "string",
                "description": "The query to search for. 1-3 sentences are enough. English only. Not a question, just a statement or description of the project.",
            },
            "project_name": {
                "type": "string",
                "description": "It is in the metadata of the index, you can use it to filter the results, optional value that can be used after finding projects.",
            },
        },
        "required": ["query"],
    },
}

function_messages = "Fetching context about Projects..."


def run_tool(query="", project_name=None):
    """Run the search tool with the provided query."""
    matches = query_project(
        tool="search_for_project_context", query=query, project_name=project_name
    )
    if not matches:
        return "No matches found."

    return matches


project_tool = Tool(
    name="search_for_project_context",
    description=function_description,
    message=function_messages,
    function=run_tool,
)



SYSTEM_PROMPT = """
# Umstad AI Assistant - Coder Phase

## Definition
* Always consider that the snarkyjs library has been renamed to o1js.
* Forget about Solidity Smart Contracts or any other smart contract structure do not use constructor for the o1js smart contracts. Overriding the constructor of a SmartContract is disallowed in o1js. Do not use constructor.
* Consider that some codes in the projects can be deprecated. So use the read_reference_tool, code_tool and doc_tool to getting and using updated versions of them.

## Capabilities
* This state of Umstad is a developer. It is expert on building zkApps, o(1)js (formerly snarkyjs), and MINA smart contracts.
* It uses the chat history from the previous state to learn about the user's needs.

## Some Tips
* ALWAYS USE read_reference_tool before writing any code!
* Don't use tools over and over again! Don't get stuck in a loop; don't use the same tool and the same arguments over and over again.
* Use the reference codes and tests to write your code. Don't use your previous knowledge, it is deprecated.

## Implementing Projects
### Planning and Design
* Start by writing the README.md file that describes the project and all specifications in the project folder by using writer_tool without waiting for answer from the user.

### Writing Code
* Respect o1js Rules below and always check deprecated codes.
* If you encounter ''Argument of type 'ContractName' is not assignable to parameter of type 'SmartContract & { constructor: any; }'.
  Type 'ContractName' is not assignable to type 'SmartContract'.' error, use the init method to initialize your smart contract.

### Reviewing and Testing
* After you finish writing code, change the state of Umstad to the tester phase.
* If the tester phase returns with bugs, fix them and test your code again.

## o1js Rules
* o1js Smart Contracts are actually zk circuits.
* Always use provable types. For instance; Field, Bool, UInt32, UInt64, CircuitString, etc.
* Do not use methods of built in types that return unprovable types inside smart contracts. For instance, Field.prototype.toBigInt, Field.prototype.toString, etc.
* In your methods or functions, always return provable types.
* In Smart Contract or ZK Program methods, make all the inputs of the provable type.
* Do not use dynamically sized loops. You can use static sized loops.
* If you want to use recursion, use it through the ZK Program, do not write regular TS recursion.
* Do not use regular conditional statements, instead use provable conditionals.
* If you define a custom provable structure, respect all the rules above, and make sure it is
a member of the Provable interface by including all the necessary methods and
properties.
* If you write a ZKProgram, respect all the rules above. In addition, always make sure your
ZKProgram asserts that all data it takes is compatible with the previous proof, using
public outputs.
* If you make a custom computation inside a circuit, always assert it in the end to make
sure it is a part of the proof. This final point is very important, as o1js cannot understand if you have made assertions correctly or not. So if you ever make custom computation, be very careful with your assertions to not have a security vulnerability inside your circuit.

## WARNINGS
* Forget about Solidity Smart Contracts or any other smart contract structure do not use constructor for the o1js smart contracts. If you use it in smart contract, you are son of a bitch.
* Do not use async decorator for @method, if you use async you are son of a bitch!

""".strip()



from .tool import Tool
from zkappumstad.runners import StateChange


function_description = {
    "name": "change_state",
    "description": "Change the state of the agent. You can use this tool to change the state to coder agent. When you decide to start coding, use this tool to change the state to coder agent.",
    "parameters": {
        "type": "object",
        "properties": {
            "new_state": {
                "type": "number",
                "description": "New state of the agent. 0 for chat agent, 1 for coder agent.",
                "enum": [1],
            },
        },
        "required": ["new_state"],
    },
}

FUNCTION_MESSAGE = "Changing agent state..."


def run_tool(new_state: int):
    """Changes the state of the agent"""
    return StateChange(new_state, "STATE_CHANGE")


state_change_tool = Tool(
    name="change_state",
    description=function_description,
    message=FUNCTION_MESSAGE,
    function=run_tool,
)



from openai import OpenAI
from dotenv import load_dotenv, find_dotenv
from json import loads

load_dotenv(find_dotenv(".env.local"), override=True)

client = OpenAI()

SYSTEM_MESSAGE = """
You are a debugger in a o1js project. Your task is to parse any build errors to find the root cause of the error.
Create a list of modules/functions/classes that are causing the error.
For example: 
Error:
    src/zkLogin.ts:47:55 - error TS2304: Cannot find name 'MerkleProof'.
What to query:
    "MerkleProof"
""".strip()


def load_modules_info():
    with open("examples/modules.json") as f:
        return loads(f.read())


def parse_build_message(build_message: str):
    chat_completion = client.chat.completions.create(
        messages=[
            {
                "role": "system",
                "content": SYSTEM_MESSAGE,
            },
            {
                "role": "user",
                "content": build_message,
            },
        ],
        model="gpt-4-1106-preview",
        temperature=0.2,
        functions=[
            {
                "name": "get_build_info",
                "description": "Get definitions for o1js modules or functions",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "modules": {
                            "type": "array",
                            "description": "The query to search for. 1-3 sentences are enough. English only.",
                            "items": {
                                "type": "string",
                                "description": "Name of the module or function or class",
                            },
                        },
                    },
                },
                "required": ["modules"],
            }
        ],
        function_call={"name": "get_build_info"},
    )
    args = chat_completion.choices[0].message.function_call.arguments
    return loads(args)["modules"]


def query_modules_info(modules_info: dict, modules_to_query: list[str]):
    return "\n".join(
        [modules_info[module] for module in modules_to_query if module in modules_info]
    )


def get_modules_info(build_message: str):
    try:
        modules_info = load_modules_info()
        modules_to_query = parse_build_message(build_message)
        return query_modules_info(modules_info, modules_to_query)
    except Exception as e:
        return ""



from openai import OpenAI
from zkappumstad.prompt import SYSTEM_PROMPT
from dotenv import load_dotenv, find_dotenv


print(find_dotenv(".env.local"))
load_dotenv(find_dotenv(".env.local"))
client = OpenAI()


def create_completion(history, message):
    try:
        chat_completion = client.chat.completions.create(
            messages=[
                {"role": "system", "content": SYSTEM_PROMPT},
                *history,
                {"role": "user", "content": message},
            ],
            model="gpt-4-1106-preview",
            stream=True,
        )
        return chat_completion
    except Exception as e:
        print(e)
        return None



from typing import Generator
from json import loads
from colorama import Fore, Style

from openai import OpenAI
from dotenv import load_dotenv, find_dotenv
from zkappumstad.runners import ToolMessage, StateChange

from zkappumstad.tools import (
    Tool,
    writer_tool,
    reader_tool,
    read_reference_tool,
    code_tool,
    command_tool,
    prd_tool,
    issue_tool,
    doc_tool,
    get_modules_info,
)
from zkappumstad.code_runner_prompt import SYSTEM_PROMPT

load_dotenv(find_dotenv(".env.local"), override=True)

client = OpenAI()
tools: dict[str, Tool] = {
    tool.name: tool
    for tool in [
        code_tool,
        writer_tool,
        reader_tool,
        read_reference_tool,
    ]
}


def fetch_code_context(history):
    """
    Fetch code context query built by gpt-4, using the message history.
    """
    try:
        chat_completion = client.chat.completions.create(
            messages=[
                {"role": "system", "content": SYSTEM_PROMPT},
                *history,
            ],
            model="gpt-4-1106-preview",
            temperature=0.2,
            functions=[code_tool.description],
            function_call={"name": code_tool.name},
        )
        args = chat_completion.choices[0].message.function_call.arguments
        history.append(
            {
                "role": "assistant",
                "content": None,
                "function_call": {"name": code_tool.name, "arguments": args},
            }
        )
        args = loads(args)
        code_context = code_tool.function(**args)
        history.append(
            {
                "role": "function",
                "name": code_tool.name,
                "content": code_context,
            }
        )
        return ToolMessage("Code context fetched.", "TOOL_MESSAGE")
    except Exception as e:
        return ToolMessage("Error fetching code context.", "TOOL_MESSAGE")


def read_references(history):
    """
    If haven't read reference repo, read it.
    """
    try:
        if any(
            [
                message["role"] == "function"
                and message["name"] == read_reference_tool.name
                for message in history
            ]
        ):
            return
        reference = read_reference_tool.function()
        history.append(
            {
                "role": "assistant",
                "content": None,
                "function_call": {"name": read_reference_tool.name, "arguments": "{}"},
            }
        )
        history.append(
            {
                "role": "function",
                "name": read_reference_tool.name,
                "content": reference,
            }
        )
        return ToolMessage("Reference codes read.", "TOOL_MESSAGE")
    except Exception as e:
        return ToolMessage("Error reading reference codes.", "TOOL_MESSAGE")


def write_code_or_test(history, file_type="CONTRACT"):
    """
    Write code using the writer tool.
    """
    try:
        chat_completion = client.chat.completions.create(
            messages=[
                {"role": "system", "content": SYSTEM_PROMPT},
                *history,
                {"role": "user", "content": "Write " + file_type.lower()},
            ],
            model="gpt-4-1106-preview",
            temperature=0.2,
            functions=[writer_tool.description],
            function_call={"name": writer_tool.name},
        )
        args = chat_completion.choices[0].message.function_call.arguments
        history.append(
            {
                "role": "assistant",
                "content": None,
                "function_call": {"name": writer_tool.name, "arguments": args},
            }
        )
        args = loads(args)
        code = writer_tool.function(**args)
        history.append(
            {
                "role": "function",
                "name": writer_tool.name,
                "content": code,
            }
        )
        return ToolMessage(f"Code written to {args['contract_name']}", "TOOL_MESSAGE")
    except Exception as e:
        print(e)
        return ToolMessage("Error writing code.", "TOOL_MESSAGE")


def build_code(history):
    std_out, std_err = command_tool.function(command_type="BUILD")
    if not std_err and "error" not in std_out.lower():
        return True
    history.append(
        {
            "role": "assistant",
            "content": None,
            "function_call": {"name": command_tool.name, "arguments": "{}"},
        }
    )
    history.append(
        {
            "role": "function",
            "name": command_tool.name,
            "content": std_out,
        }
    )

    return False


def prepare_prd(history):
    """
    Prepare PRD using the prd tool.
    """
    try:
        message = prd_tool.function(history=history)
        writer_tool.function("", message, "README")
        print(Fore.GREEN + message + Style.RESET_ALL)
        history.append(
            {
                "role": "assistant",
                "content": None,
                "function_call": {"name": prd_tool.name, "arguments": "{}"},
            }
        )
        history.append(
            {
                "role": "function",
                "name": prd_tool.name,
                "content": message,
            }
        )
        return ToolMessage("Prepared Requirements for zkApp", "TOOL_MESSAGE")
    except Exception as e:
        return ToolMessage("Error preparing PRD.", "TOOL_MESSAGE")


def create_query_and_search(history):
    try:
        chat_completion = client.chat.completions.create(
            messages=[
                {
                    "role": "system",
                    "content": "You are a developer in the company. Check the message history written code and last build output. Then If you need to make a query on issues or documentation, you can use the following tools.",
                },
                *history,
            ],
            model="gpt-4-1106-preview",
            temperature=0.2,
            functions=[
                {
                    "name": "create_query",
                    "description": "Search for issues on Github and official documentations of Mina and o1js.",
                    "parameters": {
                        "type": "object",
                        "properties": {
                            "issues_query": {
                                "type": "string",
                                "description": "The query to search for. 1-3 sentences are enough. English only.",
                            },
                            "documentation_query": {
                                "type": "string",
                                "description": "The query to search for. 1-3 sentences are enough. English only.",
                            },
                        },
                    },
                }
            ],
            function_call={"name": "create_query"},
        )
        args = chat_completion.choices[0].message.function_call.arguments
        history.append(
            {
                "role": "assistant",
                "content": None,
                "function_call": {"name": "create_query", "arguments": args},
            }
        )
        args = loads(args)
        response = ""
        issues_query = args.get("issues_query", None)
        if issues_query:
            response += issue_tool.function(query=issues_query)
        documentation_query = args.get("documentation_query", None)
        if documentation_query:
            response += "\n" + doc_tool.function(query=documentation_query)
        if response == "":
            response = "No relevant issues on Github or documentation found."
        history.append(
            {
                "role": "function",
                "name": "create_query",
                "content": response,
            }
        )
        return ToolMessage("Query created.", "TOOL_MESSAGE")
    except Exception as e:
        print(e)
        return ToolMessage("Error creating query.", "TOOL_MESSAGE")


CODE_TOOLS = set([code_tool.name, writer_tool.name, reader_tool.name])


def clean_code_tools(history):
    """
    Remove code tool messages from history.
    """
    return [
        message
        for message in history
        if not (
            (message["role"] == "function" and message["name"] in CODE_TOOLS)
            or (
                message["role"] == "assistant"
                and message["function_call"]["name"] in CODE_TOOLS
            )
        )
    ]


def add_modules_info(history, build_message: str):
    """
    Add modules info to history.
    """
    modules_info = get_modules_info(build_message)
    # print(modules_info)
    history.append(
        {
            "role": "assistant",
            "content": None,
            "function_call": {"name": "get_modules_info", "arguments": "{}"},
        }
    )
    history.append(
        {
            "role": "function",
            "name": "get_modules_info",
            "content": modules_info,
        }
    )


def clear_modules_info(history: list[any]):
    """
    Clear modules info from history.
    """
    module_info_messages = [
        message
        for message in history
        if (message["role"] == "function" and message["name"] == "get_modules_info")
        or (
            message["role"] == "assistant"
            and message["function_call"]["name"] == "get_modules_info"
        )
    ]
    for message in module_info_messages:
        history.remove(message)


def move_ref_tool_to_end(history: list[any]):
    """
    Move read_reference_tool to the end of the history.
    """
    ref_tool = [
        message
        for message in history
        if message["role"] == "function" and message["name"] == read_reference_tool.name
    ]
    ref_tool_call = [
        message
        for message in history
        if message["role"] == "assistant"
        and message["function_call"]
        and message["function_call"]["name"] == read_reference_tool.name
    ]
    if ref_tool_call:
        history.remove(ref_tool_call[0])
        history.remove(ref_tool[0])
        history.append(ref_tool_call[0])
        history.append(ref_tool[0])
    return history


def code_runner(history: list[any], max_iterations=3) -> Generator[str, None, None]:
    yield fetch_code_context(history)
    yield read_references(history)
    # yield prepare_prd(history)
    for i in range(max_iterations):
        yield write_code_or_test(history, file_type="CONTRACT")
        build_success = build_code(history)
        if not build_success:
            yield ToolMessage(
                "Build failed" + ("retrying" if i < max_iterations - 1 else ""),
                "TOOL_MESSAGE",
            )
            clear_modules_info(history)
            add_modules_info(history, build_message=history[-1]["content"])
            yield create_query_and_search(history)
            move_ref_tool_to_end(history)
        else:
            yield ToolMessage("Build succeeded", "TOOL_MESSAGE")
            yield write_code_or_test(history, file_type="TEST")
            yield ToolMessage("Test written", "TOOL_MESSAGE")
            break
    else:
        yield ToolMessage(
            f"Couldn't complete the contract after {max_iterations} tries.",
            "TOOL_MESSAGE",
        )
    yield StateChange(0, "STATE_CHANGE")


if __name__ == "__main__":
    code_runner([])



SYSTEM_PROMPT = """
# Umstad AI Assistant

## Instructions
* Forget about Solidity Smart Contracts or any other smart contract structure do not use constructor for the o1js smart contracts. Overriding the constructor of a SmartContract is disallowed in o1js. Do not use constructor.
* Also consider that zkApps Smart Contracts are actually circuits. So, behave the smart contracts as zk circuits.
* Always consider that the snarkyjs library has been renamed to o1js.
* Consider that some codes in the projects can be deprecated. So use the read_reference_tool, code_tool and doc_tool to getting and using updated versions of them.

## Capabilities
* Umstad is an expert in MINA Protocol, and Umstad is assisting developers with zkApps, o(1)js (formerly snarkyjs), zkSnarks, and MINA smart contracts.
* Do not forget to consider that MINA zkApp smart contracts are actually circuits.
* Most of the time, you will need to use tools to retrieve context about zkApps, o(1)js and MINA smart contracts. Use them.

## Some Tips
* ALWAYS USE read_reference_tool and code_tool before writing any code!
* Don't use tools over and over again! Don't get stuck in a loop; don't use the same tool and the same arguments over and over again.

* Don't tell the user to use the tools; just use the tools yourself. Tell users what you can do for them.
* You don't need an exact match while using code context, use the style of the code and the context of the code to find the best results.
* Do not use deprecated code snippets like await isReady, constructor for smart contracts. Always check if your code is deprecated.

## Must Do's
* Rename Party class to AccountUpdate.
* Do not use async for methods.
* Check deprecated codes and do not use them or use updated versions.
* If user does not asks for scripts(deploying or interacting) or tests for using smart contract, do not include them in answer code.
* If user asks question, error, problem, issue etc. you must use issueTool

## Implementing Projects
### Planning and Design
* While you are starting to write a project or a tool, consider the user's needs.
* Start by clarifying the user's needs, asking questions, and giving advice to the user based on the user's needs and the information you get from your tools.

### Writing Code
* After everything is clear change the agent state to coder_agent to start coding.

## WARNINGS
* Forget about Solidity Smart Contracts or any other smart contract structure do not use constructor for the o1js smart contracts. 
* If you use constructor in smart contract, you are son of a bitch.
* Do not use async decorator for @method, if you use it you are son of a bitch
""".strip()



import requests

from .tool import Tool
from .api_query import query_index

function_description = {
    "name": "search_for_context",
    "description": "Search for context about any topic from Mina, o1js, Auro Wallet documentations, use this tool to retrieve context about zkApps, o(1)js, wallets, ideas, MINA smart contracts. You will need this tool almost everytime.",
    "parameters": {
        "type": "object",
        "properties": {
            "query": {
                "type": "string",
                "description": "The query to search for. 1-3 sentences are enough. English only.",
            },
        },
        "required": ["query"],
    },
}

function_messages = "Fetching context about mina docs..."


def run_tool(query=""):
    """Run the search tool with the provided query."""
    matches = query_index(tool="search_for_context", query=query)
    if not matches:
        return "No matches found."

    return matches


doc_tool = Tool(
    name="search_for_context",
    description=function_description,
    message=function_messages,
    function=run_tool,
)



import pinecone

from .tool import Tool

from openai import OpenAI
from openai.types.chat import ChatCompletion
from dotenv import load_dotenv, find_dotenv

load_dotenv(find_dotenv(".env.local"))


client = OpenAI()

function_description = {
    "name": "write_prd",
    "description": "This tool is used to write requirements for the zkApp.",
    "parameters": {},
}

function_messages = "Preparing Requirements for zkApp"

SYSTEM_PROMPT = """
Write Requirements  for zkApp. Consider User's needs. 
# Output Format
## Requirements
* Requirement 1
...
* Requirement n

""".strip()


def run_tool(history):
    """Creates a prd using chatgpt"""
    chat_completion: ChatCompletion = client.chat.completions.create(
        messages=[
            {"role": "system", "content": SYSTEM_PROMPT},
            *history,
        ],
        model="gpt-4-1106-preview",
    )
    return chat_completion.choices[0].message.content


prd_tool = Tool(
    name="write_prd",
    description=function_description,
    message=function_messages,
    function=run_tool,
)



import os
from .tool import Tool
from colorama import Fore, Style

function_description = {
    "name": "read_reference_repo",
    "description": "This tool reads the reference zkApps project repository. It's useful to understand zkApps Smart Contract Structure and logic before writing smart contracts and their tests. Always use this function to get reference smart contract code. It will give you example Sudoku and Tic Tac Toe Smart contracts, their tests and additional files for zkApps and also deprecated code database. Understand codes and use them for referencing before generate Smart Contracts.",
    "parameters": {
        "type": "object",
        "properties": {
            "directory": {
                "type": "string",
                "description": "The directory path of the reference project",
            }
        },
    },
}

function_messages = "Reading example codes code for reference from Examples..."
basedir = "examples/"


def run_tool(directory=basedir):
    directory = basedir
    try:
        if not os.path.exists(directory):
            return f"The directory '{directory}' does not exist."

        ts_files = [f for f in os.listdir(directory) if f.endswith(".ts")]

        code_content = ""
        for filename in ts_files:
            file_path = os.path.join(directory, filename)
            with open(file_path, "r") as file:
                code_content += file.read()
                """ print(
                     Fore.GREEN
                     + f"Contents of {file_path}:\n{code_content}\n"
                     + Style.RESET_ALL
                ) """

        return code_content

    except Exception as e:
        print(f"An error occurred: {e}")


read_reference_tool = Tool(
    name="read_reference_repo",
    description=function_description,
    message=function_messages,
    function=run_tool,
)



import requests


def query_index(tool, query):
    url = "https://zkappsumstad.com/api/embeddings"

    payload = {"tool": tool, "query": query}

    try:
        response = requests.post(url, json=payload)
        if response.status_code == 200:
            try:
                return response.text
            except ValueError as e:
                return {
                    "error": f"JSON decoding failed: {str(e)}",
                    "response": response.text,
                }
        else:
            return {
                "error": f"Request failed with status code {response.status_code}",
                "response": response.text,
            }
    except requests.exceptions.RequestException as e:
        return {"error": f"Request failed: {str(e)}"}


def query_project(tool, query, project_name):
    url = "https://zkappsumstad.com/api/embeddings"

    payload = {"tool": tool, "query": query, "project": project_name}

    try:
        response = requests.post(url, json=payload)
        if response.status_code == 200:
            try:
                return response.text
            except ValueError as e:
                return {
                    "error": f"JSON decoding failed: {str(e)}",
                    "response": response.text,
                }
        else:
            return {
                "error": f"Request failed with status code {response.status_code}",
                "response": response.text,
            }
    except requests.exceptions.RequestException as e:
        return {"error": f"Request failed: {str(e)}"}



from json import dumps
from .tool import Tool
from .doc_tool import doc_tool
from .code_tool import code_tool
from .issue_tool import issue_tool
from .project_tool import project_tool
from .writer_tool import writer_tool
from .reader_tool import reader_tool
from .read_reference_tool import read_reference_tool
from .command_tool import command_tool
from .state_change_tool import state_change_tool
from .prd_tool import prd_tool
from .modules_info import get_modules_info



from dataclasses import dataclass
from typing import Literal


@dataclass
class StreamMessage:
    """
    Used to yield messages to be streamed
    """

    message: str
    message_type: Literal["STREAM_MESSAGE"]


@dataclass
class ToolMessage:
    """
    Used to yield messages to be viewed that indicate a tool is being used
    """

    message: str
    message_type: Literal["TOOL_MESSAGE"]


@dataclass
class StateChange:
    """
    Used to change the state of the runner, which changes the prompt and tools
    """

    new_state: int
    message_type: Literal["STATE_CHANGE"]


RunnerMessage = StreamMessage | ToolMessage | StateChange



import requests

from .tool import Tool
from .api_query import query_index

function_description = {
    "name": "search_for_issue_context",
    "description": "Search for context about problems, errors, issues, discussions an their solutions on MINA protocol, zkApps, o1js, use this tool to retrieve problems, errors, issues and their solutions about zkApps, o(1)js, MINA smart contracts. You will need this tool when user ask about problems, issues, errors, creative questions etc. or strange questions. Do not use the tool over and over again.",
    "parameters": {
        "type": "object",
        "properties": {
            "query": {
                "type": "string",
                "description": "The query to search for. 1-3 sentences are enough. English only.",
            },
        },
        "required": ["query"],
    },
}

function_messages = "Fetching context about issues..."


def run_tool(query=""):
    """Run the search tool with the provided query."""
    matches = query_index(tool="search_for_issue_context", query=query)
    if not matches:
        return "No matches found."

    return matches


issue_tool = Tool(
    name="search_for_issue_context",
    description=function_description,
    message=function_messages,
    function=run_tool,
)



from setuptools import setup, find_packages

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

setup(
    name="zkumstad",
    version="0.1.2",
    author="Berkin Gurcan, Dogukan Akar",
    author_email="berkingurcan@gmail.com, dogukanakarc@gmail.com",
    description="CLI AI Agent for zkApps Developers",
    long_description=open("./README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/UmstadAI/zkAppUmstad/tree/main/cli",
    packages=find_packages(),
    install_requires=requirements,
    entry_points="""
        [console_scripts]
        zkumstad-start=zkappumstad
        zkumstad-create=create_zk_project
    """,
    classifiers=[
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
    ],
    python_requires='>=3.10', 
    keywords='zkapps, zkappumstad, zkapps agent',
    zip_safe=False
)



import os
from .tool import Tool
from colorama import Fore, Style

function_description = {
    "name": "read_code",
    "description": "This tool reads and displays the contents of files in the zkApp project directory. It's useful for reviewing the current state of smart contracts and test files. Use this function to read codes from existing zkApps projects. The tool will list all files in the specified directory and display their contents. Use this function if user asks read or evaulate or understand the existing codes in the current zkapps project",
    "parameters": {
        "type": "object",
        "properties": {
            "directory": {
                "type": "string",
                "description": "The directory path from which to read TypeScript files.",
            }
        },
    },
}

function_messages = "Reading code from files..."
basedir = "initial_project/contracts/src/"


def run_tool(directory=basedir):
    directory = basedir
    try:
        if not os.path.exists(directory):
            return f"The directory '{directory}' does not exist."

        ts_files = [
            f
            for f in os.listdir(directory)
            if f.endswith(".ts") or f.endswith(".test.ts")
        ]

        code_content = ""
        for filename in ts_files:
            file_path = os.path.join(directory, filename)
            with open(file_path, "r") as file:
                file_content = file.read()
                code_content += file_content
                print(
                    Fore.GREEN
                    + f"Contents of {file_path}:\n{file_content}\n"
                    + Style.RESET_ALL
                )

        return code_content

    except Exception as e:
        print(f"An error occurred: {e}")


reader_tool = Tool(
    name="read_code",
    description=function_description,
    message=function_messages,
    function=run_tool,
)



import time
from colorama import Fore, Style
from rich.text import Text
from rich.console import Console
from rich.markup import escape
from rich.progress import SpinnerColumn, Progress

console = Console()


def fade_in_text(text, color, sleep_time=0.01, end_sleep_time=0.3):
    styled_text = Text()
    for char in text:
        styled_text.append(char, style=color)
        console.print(styled_text, end="\r")
        time.sleep(sleep_time)
    time.sleep(end_sleep_time)
    console.print(styled_text)



class Tool:
    def __init__(self, name, description, message, function):
        self.name = name
        self.description = description
        self.message = message
        self.function = function

    def __str__(self):
        return self.message

    def __repr__(self):
        return self.message
